package com.jollychic.monitor

import java.text.SimpleDateFormat
import java.util.Date

import com.jollychic.utils.DateUtils
import scalikejdbc.config.{DBs, DBsWithEnv}

/**
  *
  * 检查标准：
  *   对比近三天的数据变化率
  * 备注： 新建或者修改过的人群标签，将不参与数据波动检查。
  *
 */
object CheckUserGroupDataWave {

  def main(args: Array[String]): Unit = {

//    DBsWithEnv("usergroup").setupAll()
    DBs.setupAll()

    import scalikejdbc._
    // 提供有效的人群规则
    val userGroupCategoryLst: List[UserGroupCategory] = DB readOnly {
      implicit session =>
        sql"SELECT category_id,tag_based,created_time,modified_time FROM user_category  WHERE  category_status = 0"
        .map(
            result=> {// 人群规则ID
            val cid = result.long("category_id")
              // 基于标签，0：cookieId，1：userId
              val tagBased = result.long("tag_based")
              val createTime = result.string("created_time")
              val modifiedTime = result.string("modified_time")
              UserGroupCategory(cid,tagBased, createTime,modifiedTime)
            })
          .list.apply()
    }

    // 过滤出待检查波动的人群标签
    val userGroupWave = userGroupCategoryLst.filter(ugc => {
      def sub2DayMore24(first: Date, second: Date): Boolean = {
        var flag = false
        //修改时间不为null且超过1天   1000*3600*24 1day = 86400000
        if (second != null
          && System.currentTimeMillis() - second.getTime > 86400000
          && first != null
          && System.currentTimeMillis() - first.getTime > 86400000) {
          flag = true
        }
        flag
      }
      // 计算是创建或者修改的人群ID否超过一天
      sub2DayMore24(
        if (ugc.createTime != null) new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(ugc.createTime) else null ,
        if (ugc.modifiedTime != null) new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(ugc.modifiedTime) else null )
    })

    // 近三天日期时间 DateUtils
    val today = DateUtils.getNDayYMD()
    val yesterday = DateUtils.getNDayYMD(-1)

    /**
      * 计算人群标签波动率
      * @param todayCts
      * @param yesterdayCts
      * @return Double 波动率
      */
    def calculateUserGroupTagWaveRate (todayCts:Long,yesterdayCts:Long) :Double ={
      var waveRate = 0.0
      // 如今日的标签的值0 则波动值为-1
      if (todayCts == 0) waveRate = -1
      else {
        val avgCts = (todayCts + yesterdayCts )/2.0
        waveRate = (todayCts - avgCts)/todayCts
      }
      waveRate
    }

    var  errTagIdLst = List[Long]()
    // 检查波动
    userGroupWave.foreach(ugc=>{
      DBsWithEnv("monitor").setupAll()
      // 今天标签数据量
      def  calculateUserGroupTagCount (tagid:Long,date:String): Long = {
       // 定义解析函数
        val tagCount = (rs: WrappedResultSet) => rs.long("cts")
        val tagCtsOpt = DB readOnly {
          implicit session =>
            sql"  SELECT IFNULL(total_count,0) as cts  FROM tag_data_usergroup WHERE  tagid = ${tagid} AND type = 'dw_profile_tag_usergroup' AND DATE_FORMAT(update_time, '%Y-%m-%d') = ${date} ORDER BY update_time DESC LIMIT 1 "
              .map {
                tagCount
              }.single.apply()
        }
        tagCtsOpt.getOrElse[Long](0) // 提取数值
      }
      // 计算近三天标签的总量
      val todayCts = calculateUserGroupTagCount(ugc.tagId,today)
      val yesterdayCts = calculateUserGroupTagCount(ugc.tagId,yesterday)

      // 波动率
      val waveRate = calculateUserGroupTagWaveRate(todayCts,yesterdayCts)

      // 波动值超过0.2
      if (math.abs(waveRate) >= 0.2)  errTagIdLst = errTagIdLst :+ ugc.tagId

      // 提交  设置事务  同步人群标签的波动率
      DB localTx{ implicit session=>
        sql"DELETE FROM  tag_data_usergroup_day  WHERE  tagid = ${ugc.tagId} AND date = ${today}".update.apply()
        sql"INSERT INTO  tag_data_usergroup_day (tagid, date, tag_count, tag_total_per,tag_dau_per) VALUES (${ugc.tagId},${today},${todayCts},${waveRate},${waveRate})".update.apply()
      }
    })

    // 若是存在异常波动值停止Hbase同步
    if (errTagIdLst.size > 0) {
      println(
        s"""
          |+++++++++++++++++++++++++++++++++++++++
          |+++++++++++++++++++++++++++++++++++++++
          |UserGroupTag ： There may be a problem
          |  ID ： ${errTagIdLst.mkString(", | ")}
          |+++++++++++++++++++++++++++++++++++++++
          |+++++++++++++++++++++++++++++++++++++++
        """.stripMargin)
      throw new RuntimeException("UserGroupTag ： There may be a problem")
    }
  }
}




/**
 * Author: Yonggan
 * Date:   2018-08-14
 * Description:
  * @param tagId    人群ID
  * @param tagBased  标识
  * @param createTime   人群创建时间
  * @param modifiedTime 人群修改时间
  * @param todayCts     人群今日标签数量
  * @param yesterdayCts 人群昨日标签数量
  * @param beforeYesterdayCts 人群前日标签数量
 */
case class UserGroupCategory(tagId:Long, tagBased:Long, createTime:String,modifiedTime:String,
                             todayCts:Long=0,
                             yesterdayCts:Long=0,
                             beforeYesterdayCts:Long=0)
